import images from './images';

function getSvgHexagonPath(x, y, size) {
  const x1 = x;
  const y1 = y - size;
  const x2 = x + (Math.cos(Math.PI / 6) * size);
  const y2 = y - (Math.sin(Math.PI / 6) * size);
  const x3 = x + (Math.cos(Math.PI / 6) * size);
  const y3 = y + (Math.sin(Math.PI / 6) * size);
  const x4 = x;
  const y4 = y + size;
  const x5 = x - (Math.cos(Math.PI / 6) * size);
  const y5 = y + (Math.sin(Math.PI / 6) * size);
  const x6 = x - (Math.cos(Math.PI / 6) * size);
  const y6 = y - (Math.sin(Math.PI / 6) * size);

  return `M${x1},${y1}L${x2},${y2}L${x3},${y3}L${x4},${y4}L${x5},${y5}L${x6},${y6}z`;
}

let previousLinksNodes = [];
let previousNode;

function addEvents(btn, name, data) {
  btn.node.onclick = function (event) {
    event.target.dispatchEvent(
      new CustomEvent('treeSvgClick', { detail: JSON.stringify(data) })
    );

    const nodes = document.getElementById('service-tree').querySelectorAll(`path[id*=${name}-node]`);

    if(previousNode) {
      previousNode.style.fill = '#000';
    }
    nodes[0].style.fill = '#ff0000';
    previousNode = nodes[0];
    const linksNodes = document.getElementById('service-tree').querySelectorAll(`path[id*=${name}-link]`) || [];

    previousLinksNodes.forEach(n => n.style.stroke = '#000');
    linksNodes.forEach(n => n.style.stroke = '#ff0000');

    previousLinksNodes = linksNodes;
  };
}

function drawNode(paper, x, y, imageName, data = {}, nodeSize = 20) {
  const node = paper.path(getSvgHexagonPath(x + (nodeSize / 2), y, nodeSize)).attr({
    fill: '#0000FF',
    stroke: '#0000FF'
  }).rotate(30);

  node.node.setAttribute('id', `${data.name}-node`);

  if(imageName) {
    paper.image(images[imageName], x, y - (nodeSize / 2), nodeSize, nodeSize);
  }

  const btn = paper.path(getSvgHexagonPath(x + (nodeSize / 2), y, nodeSize))
    .attr({
      fill: '#000000',
      'fill-opacity': 0,
      'stroke-opacity': 0,
      title: data.name
    }).rotate(30);

  btn.node.setAttribute('id', data.name);

  btn.node.setAttribute('onmouseover', `this.dispatchEvent(
    new CustomEvent('treeSvgMouseOver', { 'detail': ${JSON.stringify(data)} })
  )`);
  btn.node.setAttribute('onmouseout', `this.dispatchEvent(
    new CustomEvent('treeSvgMouseOut', { 'detail': ${JSON.stringify(data)} })
  )`);

  addEvents(btn, data.name, data);

  return node.attr({
    text: data.name
  });
}

function drawLink(paper, id, source, target, bends = [], nodeSize = 20) {
  const b0 = bends.length > 0 ? bends[0] : source;
  const b1 = bends.length > 1 ? bends[1] : source;

  const link = paper.path(`M${source.x},${source.y}C${b0.x},${b0.y},${b1.x},${b1.y},${target.x},${target.y}`);

  link.node.setAttribute('id', id);
  return link;
}

export {
  drawNode,
  drawLink
};
